# Unity ML-Agents Toolkit Documentation

## Installation & Set-up

- [Installation](Installation.md)
  - [Using Virtual Environment](Using-Virtual-Environment.md)

## Getting Started

- [Getting Started Guide](Getting-Started.md)
- [ML-Agents Toolkit Overview](ML-Agents-Overview.md)
  - [Background: Unity](Background-Unity.md)
  - [Background: Machine Learning](Background-Machine-Learning.md)
  - [Background: PyTorch](Background-PyTorch.md)
- [Example Environments](Learning-Environment-Examples.md)

## Creating Learning Environments

- [Making a New Learning Environment](Learning-Environment-Create-New.md)
- [Designing a Learning Environment](Learning-Environment-Design.md)
  - [Designing Agents](Learning-Environment-Design-Agents.md)
- [Using an Executable Environment](Learning-Environment-Executable.md)
- [ML-Agents Package Settings](Package-Settings.md)

## Training & Inference

- [Training ML-Agents](Training-ML-Agents.md)
  - [Training Configuration File](Training-Configuration-File.md)
  - [Using TensorBoard to Observe Training](Using-Tensorboard.md)
  - [Profiling Trainers](Profiling-Python.md)
- [Sentis](Sentis.md)

## Extending ML-Agents

- [Creating Custom Side Channels](Custom-SideChannels.md)
- [Creating Custom Samplers for Environment Parameter Randomization](Training-ML-Agents.md#defining-a-new-sampler-type)

## Hugging Face Integration

- [Using Hugging Face to download and upload trained models](Hugging-Face-Integration.md)

## Python Tutorial with Google Colab

- [Using a UnityEnvironment](https://colab.research.google.com/github/Unity-Technologies/ml-agents/blob/release_22_docs/colab/Colab_UnityEnvironment_1_Run.ipynb)
- [Q-Learning with a UnityEnvironment](https://colab.research.google.com/github/Unity-Technologies/ml-agents/blob/release_22_docs/colab/Colab_UnityEnvironment_2_Train.ipynb)
- [Using Side Channels on a UnityEnvironment](https://colab.research.google.com/github/Unity-Technologies/ml-agents/blob/release_22_docs/colab/Colab_UnityEnvironment_3_SideChannel.ipynb)

## Help

- [Migrating from earlier versions of ML-Agents](Migrating.md)
- [Frequently Asked Questions](FAQ.md)
- [ML-Agents Glossary](Glossary.md)
- [Limitations](Limitations.md)

## API Docs

- [API Reference](API-Reference.md)
- [Python API Documentation](Python-LLAPI-Documentation.md)
- [How to use the Python API](Python-LLAPI.md)
- [How to use the Unity Environment Registry](Unity-Environment-Registry.md)
- [Wrapping Learning Environment as a Gym (+Baselines/Dopamine Integration)](Python-Gym-API.md)

## Translations

To make the Unity ML-Agents Toolkit accessible to the global research and Unity
developer communities, we're attempting to create and maintain translations of
our documentation. We've started with translating a subset of the documentation
to one language (Chinese), but we hope to continue translating more pages and to
other languages. Consequently, we welcome any enhancements and improvements from
the community.

- [Chinese](../localized_docs/zh-CN/)
- [Korean](../localized_docs/KR/)

## Deprecated Docs

We no longer use them ourselves and so they may not be up-to-date. We've decided
to keep them up just in case they are helpful to you.

- [Windows Anaconda Installation](Installation-Anaconda-Windows.md)
- [Using Docker](Using-Docker.md)
- [Training on the Cloud with Amazon Web Services](Training-on-Amazon-Web-Service.md)
- [Training on the Cloud with Microsoft Azure](Training-on-Microsoft-Azure.md)
- [Using the Video Recorder](https://github.com/Unity-Technologies/video-recorder)
